#!/usr/bin/python3
import argparse
import numpy as np
import re
import sys
import math


def lorentzian(freq, pole, smear):
    return 1.0/math.pi*(smear/((freq-pole)**2+smear**2))


def bose(E,T):
    #
    # Bose function
    #
    if T < 1e-10:
        return 0.0
    else:
        return math.exp(-E/T)/(1.0-math.exp(-E/T))

def boltz(E,T):
    #
    # Boltzman  function
    #
    if T < 1e-10:
        return 0.0
    else:
        return math.exp(-E/T)

ha2ev     = 27.211396132
au2kelvin = 3.1577513e5
ha2CMm1   = ha2ev*8065.5


def smearing_func(T):
        gamma0=0.003/ha2ev # Gamm0 = 3 meV
        a=0.1              # a     = 0.1
        b=0.150/ha2ev      # b     = 150 meV
        Eo= 0.025/ha2ev    # Eo    = 25 meV
        return gamma0+a*T+b*bose(Eo,T) 



"""
Build dielectric constant from excitons
Author:  C. Attaccalite
"""
#
# parse command line
#
parser = argparse.ArgumentParser(prog='make_eps',description='Build epsilon from exciton decomposition',epilog="Copyright C. Attaccalite 2018")
parser.add_argument('-f', help="exc-ph file name",type=str , default="MODE_DIP.txt", dest="excph_fname")
parser.add_argument('-ns', help="energy steps (default 1000)",type=int , default=3000, dest="ensteps")
parser.add_argument('-er', help="energy range (default [0,10] ev)",type=float , default=[5.6,6.00], dest="enrange",nargs=2)
#
# Energy of the lowest indirect exction
parser.add_argument('-ex', help="exciton energy (default 5.889 ev)",type=float , default=5.7165937, dest="exc_en")
parser.add_argument('-t', help="temperature(default 0.0 [Kelvin])",type=float , default=55.0, dest="temp")
parser.add_argument('-et', help="exc temp (default 0.0 [Kelvin])",type=float , default=75.0, dest="exc_temp")
parser.add_argument('-mu', help="chemical potential for the Bose [eV]",type=float , default=-1.0, dest="mu")
args = parser.parse_args()

print("\n * * * Build dielecrict constant from exciton analisys * * * \n\n")

args = parser.parse_args()

data=np.genfromtxt(args.excph_fname,comments="#")

delta_e=(args.enrange[1]-args.enrange[0])/float(args.ensteps)/ha2ev

temp = args.temp/au2kelvin
exc_temp = args.exc_temp/au2kelvin

print("Temperature [au] :"+str(temp))
print("Chemical potential [eV] :"+str(args.mu))

eps =np.zeros([args.ensteps,2],dtype=float)
jdos=np.zeros([args.ensteps,2],dtype=float)
lum =np.zeros([args.ensteps,2],dtype=float)
lum_tot =np.zeros([args.ensteps,2],dtype=float)

# Exciton splitting at q=(1/3,-1/6,0.)
# you can get this value from the undistorted supercell results

i1i2split=0.015/ha2ev

# Additional energy shift to match the experiments
eshift=0.24540533/ha2ev
print("Exciton split [au] :"+str(i1i2split))

for iph in range(data.shape[0]):
    for iw in range(args.ensteps):
        w=args.enrange[0]/ha2ev+delta_e*float(iw)
        eps [iw,0]=w*ha2ev
        jdos[iw,0]=w*ha2ev
        lum [iw,0]=w*ha2ev

        exc_ph_en=args.exc_en/ha2ev-data[iph][5]/ha2CMm1+eshift
        eps[iw,1] =lorentzian(w, exc_ph_en, smearing_func(temp))*data[iph][1]
        jdos[iw,1]=lorentzian(w, exc_ph_en, smearing_func(temp))
#        if args.mu != -1.0:
#            exc_en=data[ixc][0]-data[0][0]+args.mu
    
        lum[iw,1]=w*(w+2.0*data[iph][5]/ha2CMm1)**2*lorentzian(w, exc_ph_en, smearing_func(temp))*(data[iph][1]+data[iph][2])/(data[iph][5]/ha2CMm1)
        #
        # I consider only luminescence from the lowest two exctions
        # the contribution from the excitons 3,4 is unphysical becasue, in real
        # h-BN they are well separated from the 1,2 and they will not contribute,
        # See the exciton dispersion minimum to see the real splitting
        #
   #     lum[iw,1]=lum[iw,1]+w*(w+2.0*data[iph][5]/ha2CMm1)**2*lorentzian(w, exc_ph_en+i1i2split, smearing_func(temp))*(data[iph][3]+data[iph][4])/(data[iph][5]/ha2CMm1)*boltz(i1i2split,exc_temp)

    lum_tot[:,0]=lum[:,0]
    lum_tot[:,1]=lum_tot[:,1]+lum[:,1]
    np.savetxt('lum_mode_'+str(iph)+'.dat',lum,fmt='%2.15e')
        

#np.savetxt('eps_excitons.out',eps,fmt='%2.15e')
#np.savetxt('jdos_excitons.out',jdos,fmt='%2.15e')
np.savetxt('lum_excitons.out',lum_tot,fmt='%2.15e')

